import codecs
import sqlite3
from functools import partial

import source_code
from event_handler_base import EventHandlerBase
from messaging import Messaging
from source_code_event_handler import SourceCodeEventHandler
from terminado_event_handler import TerminadoEventHandler
from tornado.ioloop import IOLoop

from login_component import authorize_login
from util import create_source_code_response_data

login_component_py_response = partial(
    create_source_code_response_data, filename='login_component.py', language='python'
)


class EchoHandler(EventHandlerBase):
    def handle_event(self, anchor, data_json):
        return data_json


class Rot13Handler(EventHandlerBase):
    def handle_event(self, anchor, data_json):
        data_json['data'] = codecs.encode(data_json['data'], 'rot13')
        return data_json


class ChangeCaseHandler(EventHandlerBase):
    def handle_event(self, anchor, data_json):
        data_json['data'] = data_json['data'].upper() if data_json['data'].islower() else data_json['data'].lower()
        return data_json


class ReverseHandler(EventHandlerBase):
    def handle_event(self, anchor, data_json):
        data_json['data'] = data_json['data'][::-1]
        return data_json


class LoginHandler(EventHandlerBase):
    def handle_event(self, anchor, data_json):
        email, password = data_json['data']['email'], data_json['data']['password']
        try:
            sql_statement = source_code.find_local_variable_value(authorize_login, 'sql_statement')
            yield (
                'anchor_logger',
                'The SQL statement executed by the server will look like this:\n `{}`'.format(sql_statement)
            )

            yield ('anchor_webide',
                   login_component_py_response(content=source_code.get_source_code(authorize_login, strip_comments=False)))

            sql_statement_with_values = sql_statement.format(email, password)
            yield (
                'anchor_logger',
                'After the submitted parameters are substituted it looks like this:\n `{}`'.format(
                    sql_statement_with_values
                )
            )

            logged_in_email, is_admin = authorize_login(email, password)

            yield (
                'anchor_logger',
                'After the query is executed, it returns _{}_ as email address, and _{}_ for is_admin'.format(
                    logged_in_email, is_admin
                )
            )

            if logged_in_email is not None:
                response = 'Logged in as _{}_. You __{}have__ admin privileges.'.format(
                    logged_in_email,
                    '' if is_admin else 'don\'t '
                )
            else:
                response = 'Bad username/password!'
        except sqlite3.Warning:
            response = 'Invalid request!'

        yield ('anchor_login', '# Login page\n' + response)


if __name__ == '__main__':
    messaging = Messaging()
    anchor_a = ChangeCaseHandler('anchor_a')
    anchor_b = Rot13Handler('anchor_b')
    anchor_c = ReverseHandler('anchor_c')
    anchor_webide = SourceCodeEventHandler('anchor_webide', 'login_component.py', 'login')
    anchor_terminado = TerminadoEventHandler('anchor_terminado', 'terminado')
    IOLoop.instance().start()

